local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("AlteracValley", "AceEvent-3.0")

local landmarks = BG.landmarks

local pois = {
	right = {
		"Iceblood Tower",
		"Iceblood Graveyard",
		"Tower Point",
		"Frostwolf Graveyard",
		"East Frostwolf Tower",
		"West Frostwolf Tower",
		"Frostwolf Relief Hut",
		"Coldtooth Mine",
	},
	left = {
		"Stonehearth Bunker",
		"Stonehearth Graveyard",
		"Icewing Bunker",
		"Stormpike Graveyard",
		"Dun Baldar North Bunker",
		"Dun Baldar South Bunker",
		"Stormpike Aid Station",
		"Irondeep Mine",
	},
	top = {
		"Battleground Start",
	},
	bottom = {
		"Snowfall Graveyard",
	}
}

local engage = "The %s has engaged %s."
local reset = "The %s has reset %s."

local chatFilters = {
	["Captain Balinda Stonehearth"] = {
		["Begone, uncouth scum!  The Alliance shall prevail in Alterac Valley!"] = engage,
		["Filthy Frostwolf cowards! If you want a fight, you'll have to come to me!"] = reset,
	},
	["Captain Galvangar"] = {
		["Die!  Your kind has no place in Alterac Valley!"] = engage,
		["I'll never fall for that, fool! If you want a battle it will be on my terms and in my lair!"] = reset,
	},
	["Vanndar Stormpike"] = {
		["Soldiers of Stormpike, your General is under attack! I require aid! Come! Come! Slay these mangy Frostwolf dogs."] = engage,
		["You'll never get me out of me bunker, heathens!"] = reset,
	},
	["Drek'Thar"] = {
		["Stormpike filth! In my keep?! Slay them all!"] = engage,
		["You seek to draw the General of the Frostwolf legion out from his fortress? PREPOSTEROUS!"] = reset,
	},
}
local faction = {
	["Captain Balinda Stonehearth"] = "Horde",
	["Captain Galvangar"] = "Alliance",
	["Vanndar Stormpike"] = "Horde",
	["Drek'Thar"] = "Alliance"
}

local r_alliance = 0
local r_horde = 0

local function chat(chatFrame, msg, faction)
	local color
	if faction == "Alliance" then
		color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
	elseif faction == "Horde" then
		color = ChatTypeInfo["BG_SYSTEM_HORDE"]
	end
	chatFrame:AddMessage(msg, color.r, color.g, color.b)
end

local function filterFunc(chatFrame, event, msg, author, ...)
	if author == "Herald" then
		if msg:match("Alliance") then
			chat(chatFrame, msg, "Alliance")
		elseif msg:match("Horde") then
			chat(chatFrame, msg, "Horde")
		end
		return true -- filter
	end
	if chatFilters[author] and chatFilters[author][msg] then
		chat(chatFrame, chatFilters[author][msg]:format(faction[author], author), faction[author])
		return true
	end
	return
end

local frame

local function OnUpdate(self, elapsed)
	if not self.expire then return end
	if self.expire >= time() then
		local tleft = self.expire - time()
		local m = floor(tleft/60)
		local s = tleft % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	elseif self.switchname then
		self.text:SetText(self.switchname)
		self.switchname = nil
		self.expire = time()
		self:SetScript("OnUpdate", OnUpdateTime)
	end
end

local function OnUpdateTime(self, elapsed)
	if self.expire then
		local telapsed = time() - self.expire
		local m = floor(telapsed/60)
		local s = telapsed % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	end
end


local function getButton(orientation)
	local button = CreateFrame("Button", nil, frame)
	button:Show()
	local text = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	local time = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	local icon = button:CreateTexture(nil, "BACKGROUND")
	button:SetWidth(16)
	button:SetHeight(16)
	text:SetWidth(200)
	text:SetHeight(16)
	time:SetWidth(200)
	time:SetHeight(16)
	button.text = text
	button.time = time
	button.icon = icon
	icon:SetPoint("CENTER", button, "CENTER")
	icon:SetWidth(16)
	icon:SetHeight(16)
	icon:SetTexture("Interface\\Minimap\\POIIcons")
	button:SetHeight(16)
	if orientation == "right" then
		time:SetPoint("RIGHT", button, "LEFT")
		text:SetPoint("LEFT", button, "RIGHT")
		time:SetJustifyH("RIGHT")
		text:SetJustifyH("LEFT")
	else
		text:SetPoint("RIGHT", button, "LEFT")
		time:SetPoint("LEFT", button, "RIGHT")
		text:SetJustifyH("RIGHT")
		time:SetJustifyH("LEFT")
	end
	return button
end

local function createGUI()
	if frame then return end
	frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()
	frame.pois = {}
	frame:SetPoint("TOP", WorldStateAlwaysUpFrame, "TOP", 30, -8) -- -66)
	frame:SetWidth(260)
	frame:SetHeight(30)
	local last
	for k, v in pairs( pois.left ) do
		frame.pois[v] = getButton("left")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("RIGHT", frame, "LEFT")
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
	for k, v in pairs( pois.right ) do
		frame.pois[v] = getButton("right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("RIGHT", frame, "RIGHT")
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
	for k, v in pairs(pois.top) do
		frame.pois[v] = getButton("right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("BOTTOM", frame, "TOP", -50, 0)
		else
			frame.pois[v]:SetPoint("BOTTOM", last, "TOP")
			last = frame.pois[v]
		end
	end
	for k, v in pairs(pois.bottom) do
		frame.pois[v] = getButton("right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("TOP", frame, "TOP", -50, -56)
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
end

local function showGUI()
	if not frame then createGUI() end
	for k, v in pairs(frame.pois) do
		v.status = nil
	end
	frame.pois["Battleground Start"].icon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
	frame.pois["Battleground Start"].icon:SetTexCoord(0.57, 0.90, 0.08, 0.41)
	frame:Show()
end

local function hideGUI()
	frame:Hide()
end

function module:OnEnable()
	r_alliance = 0
	r_horde = 0
	ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
	showGUI()
	self:RegisterMessage("Battleground_POIUpdate")
	self:RegisterMessage("Battleground_StartTimer")
	self:RegisterMessage("Battleground_Started")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("UPDATE_WORLD_STATES")
	-- request sync from as many as we can
	SendAddonMessage("cap", "r", "BATTLEGROUND") -- capping
end

function module:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
	hideGUI()
end

function module:Battleground_Started(message, t)
	local poi = frame.pois["Battleground Start"]
	poi.expire = time() - t
	poi.text:SetText("Elapsed")
	poi:SetScript("OnUpdate", OnUpdateTime)
end

function module:Battleground_StartTimer(message, t)
	local poi = frame.pois["Battleground Start"]
	poi.text:SetText("Battleground Start")
	poi.switchname = "Elapsed"
	poi.expire = time() + t
	poi:SetScript("OnUpdate", OnUpdate)
end

function module:Battleground_POIUpdate(message, name, ti)
	if not frame or not frame.pois[name] then return end
	local poi = frame.pois[name]
	poi.icon:SetTexCoord( landmarks[ti][3], landmarks[ti][4], landmarks[ti][5], landmarks[ti][6])
	local status = landmarks[ti][2]
	poi:SetAlpha(1)
	poi.text:SetTextColor(.6,.6,.6)
	if status == "neutral" then
		if landmarks[ti][1] == "tower" then poi:SetAlpha(.6) end
	elseif status == "horde" then
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	elseif status == "alliance" then
		local color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	end
	if ( status == "alliance_assaulted" or status == "horde_assaulted" ) and poi.status then
		poi.expire = time() + 245
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		if status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
		poi.time:SetTextColor(color.r, color.g, color.b)
		poi:SetScript("OnUpdate", OnUpdate)
	else
		poi.status = status
		poi.expire = nil
		poi:SetScript("OnUpdate", nil)
		poi.time:SetText(nil)
	end
end

function module:CHAT_MSG_ADDON(event, prefix, message, chan, sender)
	if prefix == "cap" and message ~= "r" then
		local name, duration, elapsed, color = strmatch(message, "^(.+)@(%d+)@(%d+)@(%a+)$")
		local poi = frame.pois[name]
		if poi and not poi.expire then
			poi.expire = time() + (duration - elapsed)
			local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
			if poi.status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
			poi.time:SetTextColor(color.r, color.g, color.b)
			poi:SetScript("OnUpdate", OnUpdate)
		end
	end
end

function module:UPDATE_WORLD_STATES()
	local alliance = select(3, GetWorldStateUIInfo(1))
	local horde = select(3, GetWorldStateUIInfo(2))
	if not alliance or not horde then return end
	
	local nr = tonumber(alliance:match("([0-9]+)")) or 0
	if (r_alliance - nr) == 100 then chat(DEFAULT_CHAT_FRAME, "The Horde has slain Captain Balinda Stonehearth.", "Horde") end
	r_alliance = nr
	nr = tonumber(horde:match("([0-9]+)")) or 0
	if (r_horde - nr) == 100 then chat(DEFAULT_CHAT_FRAME, "The Alliance has slain Captain Galvangar.", "Alliance") end
	r_horde = nr
end
